//
// Created by Kirill Zhukov on 20.04.2025.
//

#ifndef SHAREDCOMMANDQUEUE_H
#define SHAREDCOMMANDQUEUE_H

#include "Command.h"
#include "utils/datastructures/LFCircullarBuffer.h"
#include <atomic>
#include <cstdint>

namespace usub::core
{
    constexpr size_t SHM_QUEUE_CAPACITY = 1024;

    class SharedCommandQueue
    {
    public:
        explicit SharedCommandQueue(size_t capacity);

        ~SharedCommandQueue();

        bool try_push(const Command& cmd);

        bool try_push_batch(const Command* cmds, size_t count);

        void finalize(Command* cmd);

        std::optional<Command> try_pop();

        size_t try_pop_batch(Command* out, size_t max_count);

    private:
        size_t capacity_;
        std::atomic<size_t> head_;
        std::atomic<size_t> tail_;
        Command* commands_;
        alignas(64) Command buffer_[];
    };
}


#endif //SHAREDCOMMANDQUEUE_H
